from protos import ydb_operation_pb2 as _ydb_operation_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AcquireResourceRequest(_message.Message):
    __slots__ = ["coordination_node_path", "operation_params", "required", "resource_path", "used"]
    COORDINATION_NODE_PATH_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    REQUIRED_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_PATH_FIELD_NUMBER: _ClassVar[int]
    USED_FIELD_NUMBER: _ClassVar[int]
    coordination_node_path: str
    operation_params: _ydb_operation_pb2.OperationParams
    required: int
    resource_path: str
    used: int
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., coordination_node_path: _Optional[str] = ..., resource_path: _Optional[str] = ..., required: _Optional[int] = ..., used: _Optional[int] = ...) -> None: ...

class AcquireResourceResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class AcquireResourceResult(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class AlterResourceRequest(_message.Message):
    __slots__ = ["coordination_node_path", "operation_params", "resource"]
    COORDINATION_NODE_PATH_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    coordination_node_path: str
    operation_params: _ydb_operation_pb2.OperationParams
    resource: Resource
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., coordination_node_path: _Optional[str] = ..., resource: _Optional[_Union[Resource, _Mapping]] = ...) -> None: ...

class AlterResourceResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class AlterResourceResult(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class CreateResourceRequest(_message.Message):
    __slots__ = ["coordination_node_path", "operation_params", "resource"]
    COORDINATION_NODE_PATH_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    coordination_node_path: str
    operation_params: _ydb_operation_pb2.OperationParams
    resource: Resource
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., coordination_node_path: _Optional[str] = ..., resource: _Optional[_Union[Resource, _Mapping]] = ...) -> None: ...

class CreateResourceResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class CreateResourceResult(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class DescribeResourceRequest(_message.Message):
    __slots__ = ["coordination_node_path", "operation_params", "resource_path"]
    COORDINATION_NODE_PATH_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_PATH_FIELD_NUMBER: _ClassVar[int]
    coordination_node_path: str
    operation_params: _ydb_operation_pb2.OperationParams
    resource_path: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., coordination_node_path: _Optional[str] = ..., resource_path: _Optional[str] = ...) -> None: ...

class DescribeResourceResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DescribeResourceResult(_message.Message):
    __slots__ = ["resource"]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    resource: Resource
    def __init__(self, resource: _Optional[_Union[Resource, _Mapping]] = ...) -> None: ...

class DropResourceRequest(_message.Message):
    __slots__ = ["coordination_node_path", "operation_params", "resource_path"]
    COORDINATION_NODE_PATH_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_PATH_FIELD_NUMBER: _ClassVar[int]
    coordination_node_path: str
    operation_params: _ydb_operation_pb2.OperationParams
    resource_path: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., coordination_node_path: _Optional[str] = ..., resource_path: _Optional[str] = ...) -> None: ...

class DropResourceResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DropResourceResult(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class HierarchicalDrrSettings(_message.Message):
    __slots__ = ["max_burst_size_coefficient", "max_units_per_second", "prefetch_coefficient", "prefetch_watermark"]
    MAX_BURST_SIZE_COEFFICIENT_FIELD_NUMBER: _ClassVar[int]
    MAX_UNITS_PER_SECOND_FIELD_NUMBER: _ClassVar[int]
    PREFETCH_COEFFICIENT_FIELD_NUMBER: _ClassVar[int]
    PREFETCH_WATERMARK_FIELD_NUMBER: _ClassVar[int]
    max_burst_size_coefficient: float
    max_units_per_second: float
    prefetch_coefficient: float
    prefetch_watermark: float
    def __init__(self, max_units_per_second: _Optional[float] = ..., max_burst_size_coefficient: _Optional[float] = ..., prefetch_coefficient: _Optional[float] = ..., prefetch_watermark: _Optional[float] = ...) -> None: ...

class ListResourcesRequest(_message.Message):
    __slots__ = ["coordination_node_path", "operation_params", "recursive", "resource_path"]
    COORDINATION_NODE_PATH_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    RECURSIVE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_PATH_FIELD_NUMBER: _ClassVar[int]
    coordination_node_path: str
    operation_params: _ydb_operation_pb2.OperationParams
    recursive: bool
    resource_path: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., coordination_node_path: _Optional[str] = ..., resource_path: _Optional[str] = ..., recursive: bool = ...) -> None: ...

class ListResourcesResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ListResourcesResult(_message.Message):
    __slots__ = ["resource_paths"]
    RESOURCE_PATHS_FIELD_NUMBER: _ClassVar[int]
    resource_paths: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, resource_paths: _Optional[_Iterable[str]] = ...) -> None: ...

class Resource(_message.Message):
    __slots__ = ["hierarchical_drr", "resource_path"]
    HIERARCHICAL_DRR_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_PATH_FIELD_NUMBER: _ClassVar[int]
    hierarchical_drr: HierarchicalDrrSettings
    resource_path: str
    def __init__(self, resource_path: _Optional[str] = ..., hierarchical_drr: _Optional[_Union[HierarchicalDrrSettings, _Mapping]] = ...) -> None: ...



import pytest
import ydb


class TestSession:
    def test_keep_in_cache_dataquery(self, driver_sync):
        # Keep prepared query for keep_in_cache only, not store query_id if client query cache disabled
        tc_settings = ydb.TableClientSettings().with_client_query_cache(enabled=False)
        table_client = ydb.TableClient(driver_sync, tc_settings)
        data_query = ydb.DataQuery("select 1", {})
        session = ydb.retry_operation_sync(lambda: table_client.session().create())
        session.transaction().execute(data_query, commit_tx=True)
        assert session.has_prepared("select 1")
        assert session._state.lookup("select 1")[1] is None
        assert session.has_prepared(data_query)
        assert session._state.lookup(data_query)[1] is None


class TestSessionPool:
    def test_checkout_from_stopped_pool(self, driver_sync):
        pool = ydb.SessionPool(driver_sync, 1)
        pool.stop()

        with pytest.raises(ValueError):
            pool.acquire()


class TestTable:
    def test_create_table_with_not_null_primary_key_by_api(self, driver_sync, database):
        table_path = database + "/test_table"

        def create_table(session: ydb.Session):
            try:
                session.drop_table(table_path)
            except ydb.issues.SchemeError:
                pass

            description = (
                ydb.TableDescription()
                .with_primary_keys("key1")
                .with_columns(
                    ydb.Column("key1", ydb.PrimitiveType.Uint64),
                    ydb.Column("value", ydb.OptionalType(ydb.PrimitiveType.Utf8)),
                )
            )

            session.create_table(table_path, description)

        with ydb.SessionPool(driver_sync) as pool:
            pool.retry_operation_sync(create_table)

        res = driver_sync.scheme_client.describe_path(table_path)
        assert res.type == ydb.scheme.SchemeEntryType.TABLE

    def test_select_text_query_with_params(self, driver_sync):
        def select(session: ydb.Session):
            text_query = "DECLARE $v AS Int64; SELECT $v"
            session.prepare(text_query)
            with session.transaction() as tx:
                tx.execute(text_query, {"$v": 1})

        pool = ydb.SessionPool(driver=driver_sync)
        pool.retry_operation_sync(select)



import os

import pytest
import ydb
from ydb import issues


def pytest_addoption(parser):
    """Add custom command line options for pytest-docker compatibility."""
    parser.addoption(
        "--docker-compose-file",
        action="store",
        default="docker-compose.yml",
        help="Path to docker-compose file (relative to project root)",
    )


@pytest.fixture(scope="session")
def docker_compose_file(pytestconfig):
    """Path to docker-compose.yml for pytest-docker."""
    compose_file = pytestconfig.getoption("--docker-compose-file")
    return os.path.join(str(pytestconfig.rootdir), compose_file)


@pytest.fixture(scope="session")
def docker_cleanup():
    """Remove volumes after tests (equivalent to --docker-compose-remove-volumes)."""
    return ["down -v --remove-orphans"]


class DockerProject:
    """Compatibility wrapper for pytest-docker-compose docker_project fixture."""

    def __init__(self, docker_compose, docker_services, endpoint):
        self._docker_compose = docker_compose
        self._docker_services = docker_services
        self._endpoint = endpoint
        self._stopped = False

    def stop(self):
        """Stop all containers (marks as stopped, actual restart happens in start())."""
        self._stopped = True
        # Use 'kill' for instant stop (simulates network failure better than graceful stop)
        self._docker_compose.execute("kill")

    def start(self):
        """Restart containers and wait for YDB to be ready."""
        if self._stopped:
            # After kill, we need to recreate the container to restore YDB properly
            self._docker_compose.execute("up -d --force-recreate")
            self._stopped = False
        else:
            self._docker_compose.execute("start")

        self._docker_services.wait_until_responsive(
            timeout=120.0,
            pause=2.0,
            check=lambda: is_ydb_responsive(self._endpoint),
        )


@pytest.fixture(scope="module")
def docker_project(docker_services, endpoint):
    """Compatibility fixture providing stop/start methods like pytest-docker-compose."""
    return DockerProject(docker_services._docker_compose, docker_services, endpoint)


def is_ydb_responsive(endpoint):
    """Check if YDB is ready to accept connections."""
    try:
        with ydb.Driver(endpoint=endpoint, database="/local") as driver:
            driver.wait(timeout=5)
            with ydb.SessionPool(driver) as pool:
                with pool.checkout() as session:
                    session.execute_scheme("create table `.sys_health/test_table` (A int32, primary key(A));")
            return True
    except Exception:
        return False


def is_ydb_secure_responsive(endpoint, root_certificates):
    """Check if YDB TLS endpoint is ready to accept connections."""
    try:
        with ydb.Driver(
            endpoint=f"grpcs://{endpoint}",
            database="/local",
            root_certificates=root_certificates,
        ) as driver:
            driver.wait(timeout=5)
            with ydb.SessionPool(driver) as pool:
                with pool.checkout() as session:
                    session.execute_scheme("create table `.sys_health/test_table_tls` (A int32, primary key(A));")
            return True
    except Exception:
        return False


@pytest.fixture(scope="module")
def endpoint(docker_ip, docker_services):
    """Wait for YDB to be responsive and return endpoint."""
    port = docker_services.port_for("ydb", 2136)
    endpoint_url = f"{docker_ip}:{port}"
    docker_services.wait_until_responsive(
        timeout=60.0,
        pause=1.0,
        check=lambda: is_ydb_responsive(endpoint_url),
    )
    yield endpoint_url


@pytest.fixture(scope="session")
def secure_endpoint(pytestconfig, docker_ip, docker_services):
    """Wait for YDB TLS endpoint to be responsive."""
    ca_path = os.path.join(str(pytestconfig.rootdir), "ydb_certs/ca.pem")

    # Wait for certificate file to be generated by container
    def wait_for_certificate():
        return os.path.exists(ca_path)

    docker_services.wait_until_responsive(
        timeout=30.0,
        pause=1.0,
        check=wait_for_certificate,
    )

    assert os.path.exists(ca_path), f"Certificate not found at {ca_path}"
    os.environ["YDB_SSL_ROOT_CERTIFICATES_FILE"] = ca_path
    root_certificates = ydb.load_ydb_root_certificate()

    port = docker_services.port_for("ydb", 2135)
    # Use 'localhost' instead of docker_ip because SSL certificate is issued for 'localhost'
    endpoint_url = f"localhost:{port}"

    docker_services.wait_until_responsive(
        timeout=60.0,
        pause=1.0,
        check=lambda: is_ydb_secure_responsive(endpoint_url, root_certificates),
    )

    yield endpoint_url


@pytest.fixture(scope="module")
def database():
    return "/local"


@pytest.fixture()
async def aio_connection(endpoint, database):
    """A fixture to wait ydb start"""
    from ydb.aio.connection import Connection
    from ydb.driver import DriverConfig

    config = DriverConfig.default_from_endpoint_and_database(endpoint, database)
    connection = Connection(endpoint, config)
    await connection.connection_ready(ready_timeout=7)
    return connection


@pytest.fixture()
async def driver(endpoint, database, event_loop):
    driver_config = ydb.DriverConfig(
        endpoint,
        database,
    )

    driver = ydb.aio.Driver(driver_config=driver_config)
    await driver.wait(timeout=15)

    yield driver

    await driver.stop(timeout=10)


@pytest.fixture()
async def driver_sync(endpoint, database, event_loop):
    driver_config = ydb.DriverConfig(
        endpoint,
        database,
    )

    driver = ydb.Driver(driver_config=driver_config)
    driver.wait(timeout=15)

    yield driver

    driver.stop(timeout=10)


@pytest.fixture()
def table_name(driver_sync, database):
    table_name = "table"

    with ydb.SessionPool(driver_sync) as pool:

        def create_table(s):
            try:
                s.drop_table(database + "/" + table_name)
            except ydb.SchemeError:
                pass

            s.execute_scheme(
                """
CREATE TABLE %s (
id Int64 NOT NULL,
i64Val Int64,
PRIMARY KEY(id)
)
"""
                % table_name
            )

        pool.retry_operation_sync(create_table)
    return table_name


@pytest.fixture()
def table_path(database, table_name) -> str:
    return database + "/" + table_name


@pytest.fixture
def column_table_name(driver_sync, database):
    table_name = "column_table"

    with ydb.SessionPool(driver_sync) as pool:

        def create_table(s):
            try:
                s.drop_table(database + "/" + table_name)
            except ydb.SchemeError:
                pass

            s.execute_scheme(
                """
CREATE TABLE %s (
id Int64 NOT NULL,
i64Val Int64,
PRIMARY KEY(id)
)
PARTITION BY HASH(id)
WITH (
    STORE = COLUMN
)
"""
                % table_name
            )

        pool.retry_operation_sync(create_table)
    return table_name


@pytest.fixture()
def column_table_path(database, column_table_name) -> str:
    return database + "/" + column_table_name


@pytest.fixture()
def topic_consumer():
    return "fixture-consumer"


@pytest.fixture()
@pytest.mark.asyncio()
async def topic_path(driver, topic_consumer, database) -> str:
    topic_path = database + "/test-topic"

    try:
        await driver.topic_client.drop_topic(topic_path)
    except issues.SchemeError:
        pass

    await driver.topic_client.create_topic(
        path=topic_path,
        consumers=[topic_consumer],
    )

    return topic_path


@pytest.fixture()
@pytest.mark.asyncio()
async def topic2_path(driver, topic_consumer, database) -> str:
    topic_path = database + "/test-topic2"

    try:
        await driver.topic_client.drop_topic(topic_path)
    except issues.SchemeError:
        pass

    await driver.topic_client.create_topic(
        path=topic_path,
        consumers=[topic_consumer],
    )

    return topic_path


@pytest.fixture()
@pytest.mark.asyncio()
async def topic_with_two_partitions_path(driver, topic_consumer, database) -> str:
    topic_path = database + "/test-topic-two-partitions"

    try:
        await driver.topic_client.drop_topic(topic_path)
    except issues.SchemeError:
        pass

    await driver.topic_client.create_topic(
        path=topic_path,
        consumers=[topic_consumer],
        min_active_partitions=2,
        partition_count_limit=2,
    )

    return topic_path


@pytest.fixture()
@pytest.mark.asyncio()
async def topic_with_messages(driver, topic_consumer, database):
    topic_path = database + "/test-topic-with-messages"
    try:
        await driver.topic_client.drop_topic(topic_path)
    except issues.SchemeError:
        pass

    await driver.topic_client.create_topic(
        path=topic_path,
        consumers=[topic_consumer],
    )

    writer = driver.topic_client.writer(topic_path, producer_id="fixture-producer-id", codec=ydb.TopicCodec.RAW)
    await writer.write_with_ack(
        [
            ydb.TopicWriterMessage(data="123".encode()),
            ydb.TopicWriterMessage(data="456".encode()),
        ]
    )
    await writer.write_with_ack(
        [
            ydb.TopicWriterMessage(data="789".encode()),
            ydb.TopicWriterMessage(data="0".encode()),
        ]
    )
    await writer.close()
    return topic_path


@pytest.fixture()
@pytest.mark.asyncio()
async def topic_with_messages_with_metadata(driver, topic_consumer, database):
    topic_path = database + "/test-topic-with-messages-with-metadata"
    try:
        await driver.topic_client.drop_topic(topic_path)
    except issues.SchemeError:
        pass

    await driver.topic_client.create_topic(
        path=topic_path,
        consumers=[topic_consumer],
    )

    writer = driver.topic_client.writer(topic_path, producer_id="fixture-producer-id", codec=ydb.TopicCodec.RAW)
    await writer.write_with_ack(
        [
            ydb.TopicWriterMessage(data="123".encode(), metadata_items={"key": "value"}),
            ydb.TopicWriterMessage(data="456".encode(), metadata_items={"key": b"value"}),
        ]
    )
    await writer.close()
    return topic_path


@pytest.fixture()
@pytest.mark.asyncio()
async def topic_reader(driver, topic_consumer, topic_path) -> ydb.TopicReaderAsyncIO:
    reader = driver.topic_client.reader(topic=topic_path, consumer=topic_consumer)
    yield reader
    await reader.close()


@pytest.fixture()
def topic_reader_sync(driver_sync, topic_consumer, topic_path) -> ydb.TopicReader:
    reader = driver_sync.topic_client.reader(topic=topic_path, consumer=topic_consumer)
    yield reader
    reader.close()



import pytest
import ydb.aio


@pytest.mark.asyncio
class TestSessionPool:
    async def test_checkout_from_stopped_pool(self, driver):
        pool = ydb.aio.SessionPool(driver, 1)
        await pool.stop()

        with pytest.raises(ValueError):
            await pool.acquire()



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: draft/protos/ydb_maintenance.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'draft/protos/ydb_maintenance.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v6.protos.annotations import validation_pb2 as protos_dot_annotations_dot_validation__pb2
from ydb._grpc.v6.protos import ydb_status_codes_pb2 as protos_dot_ydb__status__codes__pb2
from ydb._grpc.v6.protos import ydb_discovery_pb2 as protos_dot_ydb__discovery__pb2
from ydb._grpc.v6.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"draft/protos/ydb_maintenance.proto\x12\x0fYdb.Maintenance\x1a#protos/annotations/validation.proto\x1a\x1dprotos/ydb_status_codes.proto\x1a\x1aprotos/ydb_discovery.proto\x1a\x1aprotos/ydb_operation.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xf0\x02\n\x04Node\x12\x0f\n\x07node_id\x18\x01 \x01(\r\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\r\x12-\n\x08location\x18\x04 \x01(\x0b\x32\x1b.Ydb.Discovery.NodeLocation\x12)\n\x05state\x18\x05 \x01(\x0e\x32\x1a.Ydb.Maintenance.ItemState\x12\x34\n\x07storage\x18\x06 \x01(\x0b\x32!.Ydb.Maintenance.Node.StorageNodeH\x00\x12\x34\n\x07\x64ynamic\x18\x07 \x01(\x0b\x32!.Ydb.Maintenance.Node.DynamicNodeH\x00\x12.\n\nstart_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07version\x18\t \x01(\t\x1a\r\n\x0bStorageNode\x1a\x1d\n\x0b\x44ynamicNode\x12\x0e\n\x06tenant\x18\x01 \x01(\tB\x06\n\x04type\"T\n\x17ListClusterNodesRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\">\n\x16ListClusterNodesResult\x12$\n\x05nodes\x18\x01 \x03(\x0b\x32\x15.Ydb.Maintenance.Node\"H\n\x18ListClusterNodesResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xc3\x01\n\x16MaintenanceTaskOptions\x12\x19\n\x08task_uid\x18\x01 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x01\x12\x1c\n\x0b\x64\x65scription\x18\x02 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x01\x12<\n\x11\x61vailability_mode\x18\x03 \x01(\x0e\x32!.Ydb.Maintenance.AvailabilityMode\x12\x0f\n\x07\x64ry_run\x18\x04 \x01(\x08\x12!\n\x08priority\x18\x05 \x01(\x05\x42\x0f\xb2\xe6*\x0b[-100; 100]\"B\n\x0b\x41\x63tionScope\x12\x11\n\x07node_id\x18\x01 \x01(\rH\x00\x12\x17\n\x04host\x18\x02 \x01(\tB\x07\xa2\xe6*\x03\x18\xff\x01H\x00\x42\x07\n\x05scope\"f\n\nLockAction\x12+\n\x05scope\x18\x01 \x01(\x0b\x32\x1c.Ydb.Maintenance.ActionScope\x12+\n\x08\x64uration\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\"F\n\x06\x41\x63tion\x12\x32\n\x0block_action\x18\x01 \x01(\x0b\x32\x1b.Ydb.Maintenance.LockActionH\x00\x42\x08\n\x06\x61\x63tion\"?\n\x0b\x41\x63tionGroup\x12\x30\n\x07\x61\x63tions\x18\x01 \x03(\x0b\x32\x17.Ydb.Maintenance.ActionB\x06\x9a\xe6*\x02(\x01\"\xd5\x01\n\x1c\x43reateMaintenanceTaskRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12=\n\x0ctask_options\x18\x02 \x01(\x0b\x32\'.Ydb.Maintenance.MaintenanceTaskOptions\x12;\n\raction_groups\x18\x03 \x03(\x0b\x32\x1c.Ydb.Maintenance.ActionGroupB\x06\x9a\xe6*\x02(\x01\"u\n\x1dRefreshMaintenanceTaskRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x19\n\x08task_uid\x18\x02 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x01\"]\n\tActionUid\x12\x19\n\x08task_uid\x18\x01 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x01\x12\x19\n\x08group_id\x18\x02 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x01\x12\x1a\n\taction_id\x18\x03 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x01\"\x8f\x06\n\x0b\x41\x63tionState\x12\'\n\x06\x61\x63tion\x18\x01 \x01(\x0b\x32\x17.Ydb.Maintenance.Action\x12.\n\naction_uid\x18\x02 \x01(\x0b\x32\x1a.Ydb.Maintenance.ActionUid\x12\x39\n\x06status\x18\x03 \x01(\x0e\x32).Ydb.Maintenance.ActionState.ActionStatus\x12\x39\n\x06reason\x18\x04 \x01(\x0e\x32).Ydb.Maintenance.ActionState.ActionReason\x12\x16\n\x0ereason_details\x18\x06 \x01(\t\x12,\n\x08\x64\x65\x61\x64line\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"e\n\x0c\x41\x63tionStatus\x12\x1d\n\x19\x41\x43TION_STATUS_UNSPECIFIED\x10\x00\x12\x19\n\x15\x41\x43TION_STATUS_PENDING\x10\x01\x12\x1b\n\x17\x41\x43TION_STATUS_PERFORMED\x10\x02\"\x83\x03\n\x0c\x41\x63tionReason\x12\x1d\n\x19\x41\x43TION_REASON_UNSPECIFIED\x10\x00\x12\x14\n\x10\x41\x43TION_REASON_OK\x10\x01\x12-\n)ACTION_REASON_TOO_MANY_UNAVAILABLE_VDISKS\x10\x02\x12:\n6ACTION_REASON_TOO_MANY_UNAVAILABLE_STATE_STORAGE_RINGS\x10\x03\x12.\n*ACTION_REASON_DISABLED_NODES_LIMIT_REACHED\x10\x04\x12\x35\n1ACTION_REASON_TENANT_DISABLED_NODES_LIMIT_REACHED\x10\x05\x12\x1f\n\x1b\x41\x43TION_REASON_WRONG_REQUEST\x10\x06\x12\x30\n,ACTION_REASON_SYS_TABLETS_NODE_LIMIT_REACHED\x10\x07\x12\x19\n\x15\x41\x43TION_REASON_GENERIC\x10\x08\"H\n\x11\x41\x63tionGroupStates\x12\x33\n\raction_states\x18\x01 \x03(\x0b\x32\x1c.Ydb.Maintenance.ActionState\"\xb0\x01\n\x15MaintenanceTaskResult\x12\x10\n\x08task_uid\x18\x01 \x01(\t\x12?\n\x13\x61\x63tion_group_states\x18\x02 \x03(\x0b\x32\".Ydb.Maintenance.ActionGroupStates\x12\x34\n\x0bretry_after\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x42\x0e\n\x0c_retry_after\"G\n\x17MaintenanceTaskResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"q\n\x19GetMaintenanceTaskRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x19\n\x08task_uid\x18\x02 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x01\"\x9a\x01\n\x18GetMaintenanceTaskResult\x12=\n\x0ctask_options\x18\x01 \x01(\x0b\x32\'.Ydb.Maintenance.MaintenanceTaskOptions\x12?\n\x13\x61\x63tion_group_states\x18\x02 \x03(\x0b\x32\".Ydb.Maintenance.ActionGroupStates\"J\n\x1aGetMaintenanceTaskResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"t\n\x1bListMaintenanceTasksRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x11\n\x04user\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x07\n\x05_user\"0\n\x1aListMaintenanceTasksResult\x12\x12\n\ntasks_uids\x18\x01 \x03(\t\"L\n\x1cListMaintenanceTasksResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"r\n\x1a\x44ropMaintenanceTaskRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x19\n\x08task_uid\x18\x02 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x01\"M\n\x1dManageMaintenanceTaskResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x8b\x01\n\x15\x43ompleteActionRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x37\n\x0b\x61\x63tion_uids\x18\x02 \x03(\x0b\x32\x1a.Ydb.Maintenance.ActionUidB\x06\x9a\xe6*\x02(\x01\"\xbe\x01\n\x12ManageActionResult\x12\x43\n\x0f\x61\x63tion_statuses\x18\x01 \x03(\x0b\x32*.Ydb.Maintenance.ManageActionResult.Status\x1a\x63\n\x06Status\x12.\n\naction_uid\x18\x01 \x01(\x0b\x32\x1a.Ydb.Maintenance.ActionUid\x12)\n\x06status\x18\x02 \x01(\x0e\x32\x19.Ydb.StatusIds.StatusCode\"D\n\x14ManageActionResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation*k\n\tItemState\x12\x1a\n\x16ITEM_STATE_UNSPECIFIED\x10\x00\x12\x11\n\rITEM_STATE_UP\x10\x01\x12\x1a\n\x16ITEM_STATE_MAINTENANCE\x10\x02\x12\x13\n\x0fITEM_STATE_DOWN\x10\x03*\x8c\x01\n\x10\x41vailabilityMode\x12!\n\x1d\x41VAILABILITY_MODE_UNSPECIFIED\x10\x00\x12\x1c\n\x18\x41VAILABILITY_MODE_STRONG\x10\x01\x12\x1a\n\x16\x41VAILABILITY_MODE_WEAK\x10\x02\x12\x1b\n\x17\x41VAILABILITY_MODE_FORCE\x10\x03\x42n\n#tech.ydb.proto.draft.maintenance.v1ZDgithub.com/ydb-platform/ydb-go-genproto/draft/protos/Ydb_Maintenance\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'draft.protos.ydb_maintenance_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n#tech.ydb.proto.draft.maintenance.v1ZDgithub.com/ydb-platform/ydb-go-genproto/draft/protos/Ydb_Maintenance\370\001\001'
  _globals['_MAINTENANCETASKOPTIONS'].fields_by_name['task_uid']._loaded_options = None
  _globals['_MAINTENANCETASKOPTIONS'].fields_by_name['task_uid']._serialized_options = b'\242\346*\003\030\200\001'
  _globals['_MAINTENANCETASKOPTIONS'].fields_by_name['description']._loaded_options = None
  _globals['_MAINTENANCETASKOPTIONS'].fields_by_name['description']._serialized_options = b'\242\346*\003\030\200\001'
  _globals['_MAINTENANCETASKOPTIONS'].fields_by_name['priority']._loaded_options = None
  _globals['_MAINTENANCETASKOPTIONS'].fields_by_name['priority']._serialized_options = b'\262\346*\013[-100; 100]'
  _globals['_ACTIONSCOPE'].fields_by_name['host']._loaded_options = None
  _globals['_ACTIONSCOPE'].fields_by_name['host']._serialized_options = b'\242\346*\003\030\377\001'
  _globals['_ACTIONGROUP'].fields_by_name['actions']._loaded_options = None
  _globals['_ACTIONGROUP'].fields_by_name['actions']._serialized_options = b'\232\346*\002(\001'
  _globals['_CREATEMAINTENANCETASKREQUEST'].fields_by_name['action_groups']._loaded_options = None
  _globals['_CREATEMAINTENANCETASKREQUEST'].fields_by_name['action_groups']._serialized_options = b'\232\346*\002(\001'
  _globals['_REFRESHMAINTENANCETASKREQUEST'].fields_by_name['task_uid']._loaded_options = None
  _globals['_REFRESHMAINTENANCETASKREQUEST'].fields_by_name['task_uid']._serialized_options = b'\242\346*\003\030\200\001'
  _globals['_ACTIONUID'].fields_by_name['task_uid']._loaded_options = None
  _globals['_ACTIONUID'].fields_by_name['task_uid']._serialized_options = b'\242\346*\003\030\200\001'
  _globals['_ACTIONUID'].fields_by_name['group_id']._loaded_options = None
  _globals['_ACTIONUID'].fields_by_name['group_id']._serialized_options = b'\242\346*\003\030\200\001'
  _globals['_ACTIONUID'].fields_by_name['action_id']._loaded_options = None
  _globals['_ACTIONUID'].fields_by_name['action_id']._serialized_options = b'\242\346*\003\030\200\001'
  _globals['_GETMAINTENANCETASKREQUEST'].fields_by_name['task_uid']._loaded_options = None
  _globals['_GETMAINTENANCETASKREQUEST'].fields_by_name['task_uid']._serialized_options = b'\242\346*\003\030\200\001'
  _globals['_DROPMAINTENANCETASKREQUEST'].fields_by_name['task_uid']._loaded_options = None
  _globals['_DROPMAINTENANCETASKREQUEST'].fields_by_name['task_uid']._serialized_options = b'\242\346*\003\030\200\001'
  _globals['_COMPLETEACTIONREQUEST'].fields_by_name['action_uids']._loaded_options = None
  _globals['_COMPLETEACTIONREQUEST'].fields_by_name['action_uids']._serialized_options = b'\232\346*\002(\001'
  _globals['_ITEMSTATE']._serialized_start=4082
  _globals['_ITEMSTATE']._serialized_end=4189
  _globals['_AVAILABILITYMODE']._serialized_start=4192
  _globals['_AVAILABILITYMODE']._serialized_end=4332
  _globals['_NODE']._serialized_start=245
  _globals['_NODE']._serialized_end=613
  _globals['_NODE_STORAGENODE']._serialized_start=561
  _globals['_NODE_STORAGENODE']._serialized_end=574
  _globals['_NODE_DYNAMICNODE']._serialized_start=576
  _globals['_NODE_DYNAMICNODE']._serialized_end=605
  _globals['_LISTCLUSTERNODESREQUEST']._serialized_start=615
  _globals['_LISTCLUSTERNODESREQUEST']._serialized_end=699
  _globals['_LISTCLUSTERNODESRESULT']._serialized_start=701
  _globals['_LISTCLUSTERNODESRESULT']._serialized_end=763
  _globals['_LISTCLUSTERNODESRESPONSE']._serialized_start=765
  _globals['_LISTCLUSTERNODESRESPONSE']._serialized_end=837
  _globals['_MAINTENANCETASKOPTIONS']._serialized_start=840
  _globals['_MAINTENANCETASKOPTIONS']._serialized_end=1035
  _globals['_ACTIONSCOPE']._serialized_start=1037
  _globals['_ACTIONSCOPE']._serialized_end=1103
  _globals['_LOCKACTION']._serialized_start=1105
  _globals['_LOCKACTION']._serialized_end=1207
  _globals['_ACTION']._serialized_start=1209
  _globals['_ACTION']._serialized_end=1279
  _globals['_ACTIONGROUP']._serialized_start=1281
  _globals['_ACTIONGROUP']._serialized_end=1344
  _globals['_CREATEMAINTENANCETASKREQUEST']._serialized_start=1347
  _globals['_CREATEMAINTENANCETASKREQUEST']._serialized_end=1560
  _globals['_REFRESHMAINTENANCETASKREQUEST']._serialized_start=1562
  _globals['_REFRESHMAINTENANCETASKREQUEST']._serialized_end=1679
  _globals['_ACTIONUID']._serialized_start=1681
  _globals['_ACTIONUID']._serialized_end=1774
  _globals['_ACTIONSTATE']._serialized_start=1777
  _globals['_ACTIONSTATE']._serialized_end=2560
  _globals['_ACTIONSTATE_ACTIONSTATUS']._serialized_start=2069
  _globals['_ACTIONSTATE_ACTIONSTATUS']._serialized_end=2170
  _globals['_ACTIONSTATE_ACTIONREASON']._serialized_start=2173
  _globals['_ACTIONSTATE_ACTIONREASON']._serialized_end=2560
  _globals['_ACTIONGROUPSTATES']._serialized_start=2562
  _globals['_ACTIONGROUPSTATES']._serialized_end=2634
  _globals['_MAINTENANCETASKRESULT']._serialized_start=2637
  _globals['_MAINTENANCETASKRESULT']._serialized_end=2813
  _globals['_MAINTENANCETASKRESPONSE']._serialized_start=2815
  _globals['_MAINTENANCETASKRESPONSE']._serialized_end=2886
  _globals['_GETMAINTENANCETASKREQUEST']._serialized_start=2888
  _globals['_GETMAINTENANCETASKREQUEST']._serialized_end=3001
  _globals['_GETMAINTENANCETASKRESULT']._serialized_start=3004
  _globals['_GETMAINTENANCETASKRESULT']._serialized_end=3158
  _globals['_GETMAINTENANCETASKRESPONSE']._serialized_start=3160
  _globals['_GETMAINTENANCETASKRESPONSE']._serialized_end=3234
  _globals['_LISTMAINTENANCETASKSREQUEST']._serialized_start=3236
  _globals['_LISTMAINTENANCETASKSREQUEST']._serialized_end=3352
  _globals['_LISTMAINTENANCETASKSRESULT']._serialized_start=3354
  _globals['_LISTMAINTENANCETASKSRESULT']._serialized_end=3402
  _globals['_LISTMAINTENANCETASKSRESPONSE']._serialized_start=3404
  _globals['_LISTMAINTENANCETASKSRESPONSE']._serialized_end=3480
  _globals['_DROPMAINTENANCETASKREQUEST']._serialized_start=3482
  _globals['_DROPMAINTENANCETASKREQUEST']._serialized_end=3596
  _globals['_MANAGEMAINTENANCETASKRESPONSE']._serialized_start=3598
  _globals['_MANAGEMAINTENANCETASKRESPONSE']._serialized_end=3675
  _globals['_COMPLETEACTIONREQUEST']._serialized_start=3678
  _globals['_COMPLETEACTIONREQUEST']._serialized_end=3817
  _globals['_MANAGEACTIONRESULT']._serialized_start=3820
  _globals['_MANAGEACTIONRESULT']._serialized_end=4010
  _globals['_MANAGEACTIONRESULT_STATUS']._serialized_start=3911
  _globals['_MANAGEACTIONRESULT_STATUS']._serialized_end=4010
  _globals['_MANAGEACTIONRESPONSE']._serialized_start=4012
  _globals['_MANAGEACTIONRESPONSE']._serialized_end=4080
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb_cms_v1.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v5.protos import ydb_cms_pb2 as protos_dot_ydb__cms__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10ydb_cms_v1.proto\x12\nYdb.Cms.V1\x1a\x14protos/ydb_cms.proto2\x9c\x04\n\nCmsService\x12Q\n\x0e\x43reateDatabase\x12\x1e.Ydb.Cms.CreateDatabaseRequest\x1a\x1f.Ydb.Cms.CreateDatabaseResponse\x12Z\n\x11GetDatabaseStatus\x12!.Ydb.Cms.GetDatabaseStatusRequest\x1a\".Ydb.Cms.GetDatabaseStatusResponse\x12N\n\rAlterDatabase\x12\x1d.Ydb.Cms.AlterDatabaseRequest\x1a\x1e.Ydb.Cms.AlterDatabaseResponse\x12N\n\rListDatabases\x12\x1d.Ydb.Cms.ListDatabasesRequest\x1a\x1e.Ydb.Cms.ListDatabasesResponse\x12Q\n\x0eRemoveDatabase\x12\x1e.Ydb.Cms.RemoveDatabaseRequest\x1a\x1f.Ydb.Cms.RemoveDatabaseResponse\x12l\n\x17\x44\x65scribeDatabaseOptions\x12\'.Ydb.Cms.DescribeDatabaseOptionsRequest\x1a(.Ydb.Cms.DescribeDatabaseOptionsResponseBK\n\x15tech.ydb.proto.cms.v1Z2github.com/ydb-platform/ydb-go-genproto/Ydb_Cms_V1b\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb_cms_v1_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025tech.ydb.proto.cms.v1Z2github.com/ydb-platform/ydb-go-genproto/Ydb_Cms_V1'
  _CMSSERVICE._serialized_start=55
  _CMSSERVICE._serialized_end=595
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from enum import IntEnum
import typing

if typing.TYPE_CHECKING:
    from ..v4.protos import ydb_coordination_pb2
else:
    from ..common.protos import ydb_coordination_pb2


class ConsistencyMode(IntEnum):
    UNSET = ydb_coordination_pb2.ConsistencyMode.CONSISTENCY_MODE_UNSET
    STRICT = ydb_coordination_pb2.ConsistencyMode.CONSISTENCY_MODE_STRICT
    RELAXED = ydb_coordination_pb2.ConsistencyMode.CONSISTENCY_MODE_RELAXED


class RateLimiterCountersMode(IntEnum):
    UNSET = ydb_coordination_pb2.RateLimiterCountersMode.RATE_LIMITER_COUNTERS_MODE_UNSET
    AGGREGATED = ydb_coordination_pb2.RateLimiterCountersMode.RATE_LIMITER_COUNTERS_MODE_AGGREGATED
    DETAILED = ydb_coordination_pb2.RateLimiterCountersMode.RATE_LIMITER_COUNTERS_MODE_DETAILED


@dataclass
class NodeConfig:
    attach_consistency_mode: ConsistencyMode
    rate_limiter_counters_mode: RateLimiterCountersMode
    read_consistency_mode: ConsistencyMode
    self_check_period_millis: int
    session_grace_period_millis: int

    @staticmethod
    def from_proto(msg: ydb_coordination_pb2.Config) -> "NodeConfig":
        return NodeConfig(
            attach_consistency_mode=msg.attach_consistency_mode,
            rate_limiter_counters_mode=msg.rate_limiter_counters_mode,
            read_consistency_mode=msg.read_consistency_mode,
            self_check_period_millis=msg.self_check_period_millis,
            session_grace_period_millis=msg.session_grace_period_millis,
        )

    def to_proto(self) -> ydb_coordination_pb2.Config:
        return ydb_coordination_pb2.Config(
            attach_consistency_mode=self.attach_consistency_mode,
            rate_limiter_counters_mode=self.rate_limiter_counters_mode,
            read_consistency_mode=self.read_consistency_mode,
            self_check_period_millis=self.self_check_period_millis,
            session_grace_period_millis=self.session_grace_period_millis,
        )


class DescribeResult:
    @staticmethod
    def from_proto(msg: ydb_coordination_pb2.DescribeNodeResponse) -> "NodeConfig":
        result = ydb_coordination_pb2.DescribeNodeResult()
        msg.operation.result.Unpack(result)
        return NodeConfig.from_proto(result.config)


@dataclass
class AcquireSemaphoreResult:
    req_id: int
    acquired: bool
    status: int

    @staticmethod
    def from_proto(msg: ydb_coordination_pb2.SessionResponse.AcquireSemaphoreResult) -> "AcquireSemaphoreResult":
        return AcquireSemaphoreResult(
            req_id=msg.req_id,
            acquired=msg.acquired,
            status=msg.status,
        )


@dataclass
class CreateSemaphoreResult:
    req_id: int
    status: int

    @staticmethod
    def from_proto(msg: ydb_coordination_pb2.SessionResponse.CreateSemaphoreResult) -> "CreateSemaphoreResult":
        return CreateSemaphoreResult(
            req_id=msg.req_id,
            status=msg.status,
        )


@dataclass
class DescribeLockResult:
    req_id: int
    status: int
    watch_added: bool
    count: int
    data: bytes
    ephemeral: bool
    limit: int
    name: str
    owners: list
    waiters: list

    @staticmethod
    def from_proto(msg: ydb_coordination_pb2.SessionResponse.DescribeSemaphoreResult) -> "DescribeLockResult":
        return DescribeLockResult(
            req_id=msg.req_id,
            status=msg.status,
            watch_added=msg.watch_added,
            count=msg.semaphore_description.count,
            data=msg.semaphore_description.data,
            ephemeral=msg.semaphore_description.ephemeral,
            limit=msg.semaphore_description.limit,
            name=msg.semaphore_description.name,
            owners=msg.semaphore_description.owners,
            waiters=msg.semaphore_description.waiters,
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from ydb._grpc.v6.protos import ydb_rate_limiter_pb2 as protos_dot_ydb__rate__limiter__pb2

GRPC_GENERATED_VERSION = '1.76.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in ydb_rate_limiter_v1_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class RateLimiterServiceStub(object):
    """Service that implements distributed rate limiting.

    To use rate limiter functionality you need an existing coordination node.

    Control plane API
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateResource = channel.unary_unary(
                '/Ydb.RateLimiter.V1.RateLimiterService/CreateResource',
                request_serializer=protos_dot_ydb__rate__limiter__pb2.CreateResourceRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__rate__limiter__pb2.CreateResourceResponse.FromString,
                _registered_method=True)
        self.AlterResource = channel.unary_unary(
                '/Ydb.RateLimiter.V1.RateLimiterService/AlterResource',
                request_serializer=protos_dot_ydb__rate__limiter__pb2.AlterResourceRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__rate__limiter__pb2.AlterResourceResponse.FromString,
                _registered_method=True)
        self.DropResource = channel.unary_unary(
                '/Ydb.RateLimiter.V1.RateLimiterService/DropResource',
                request_serializer=protos_dot_ydb__rate__limiter__pb2.DropResourceRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__rate__limiter__pb2.DropResourceResponse.FromString,
                _registered_method=True)
        self.ListResources = channel.unary_unary(
                '/Ydb.RateLimiter.V1.RateLimiterService/ListResources',
                request_serializer=protos_dot_ydb__rate__limiter__pb2.ListResourcesRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__rate__limiter__pb2.ListResourcesResponse.FromString,
                _registered_method=True)
        self.DescribeResource = channel.unary_unary(
                '/Ydb.RateLimiter.V1.RateLimiterService/DescribeResource',
                request_serializer=protos_dot_ydb__rate__limiter__pb2.DescribeResourceRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__rate__limiter__pb2.DescribeResourceResponse.FromString,
                _registered_method=True)
        self.AcquireResource = channel.unary_unary(
                '/Ydb.RateLimiter.V1.RateLimiterService/AcquireResource',
                request_serializer=protos_dot_ydb__rate__limiter__pb2.AcquireResourceRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__rate__limiter__pb2.AcquireResourceResponse.FromString,
                _registered_method=True)


class RateLimiterServiceServicer(object):
    """Service that implements distributed rate limiting.

    To use rate limiter functionality you need an existing coordination node.

    Control plane API
    """

    def CreateResource(self, request, context):
        """Create a new resource in existing coordination node.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AlterResource(self, request, context):
        """Update a resource in coordination node.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropResource(self, request, context):
        """Delete a resource from coordination node.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListResources(self, request, context):
        """List resources in given coordination node.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeResource(self, request, context):
        """Describe properties of resource in coordination node.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AcquireResource(self, request, context):
        """Take units for usage of a resource in coordination node.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RateLimiterServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateResource': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateResource,
                    request_deserializer=protos_dot_ydb__rate__limiter__pb2.CreateResourceRequest.FromString,
                    response_serializer=protos_dot_ydb__rate__limiter__pb2.CreateResourceResponse.SerializeToString,
            ),
            'AlterResource': grpc.unary_unary_rpc_method_handler(
                    servicer.AlterResource,
                    request_deserializer=protos_dot_ydb__rate__limiter__pb2.AlterResourceRequest.FromString,
                    response_serializer=protos_dot_ydb__rate__limiter__pb2.AlterResourceResponse.SerializeToString,
            ),
            'DropResource': grpc.unary_unary_rpc_method_handler(
                    servicer.DropResource,
                    request_deserializer=protos_dot_ydb__rate__limiter__pb2.DropResourceRequest.FromString,
                    response_serializer=protos_dot_ydb__rate__limiter__pb2.DropResourceResponse.SerializeToString,
            ),
            'ListResources': grpc.unary_unary_rpc_method_handler(
                    servicer.ListResources,
                    request_deserializer=protos_dot_ydb__rate__limiter__pb2.ListResourcesRequest.FromString,
                    response_serializer=protos_dot_ydb__rate__limiter__pb2.ListResourcesResponse.SerializeToString,
            ),
            'DescribeResource': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeResource,
                    request_deserializer=protos_dot_ydb__rate__limiter__pb2.DescribeResourceRequest.FromString,
                    response_serializer=protos_dot_ydb__rate__limiter__pb2.DescribeResourceResponse.SerializeToString,
            ),
            'AcquireResource': grpc.unary_unary_rpc_method_handler(
                    servicer.AcquireResource,
                    request_deserializer=protos_dot_ydb__rate__limiter__pb2.AcquireResourceRequest.FromString,
                    response_serializer=protos_dot_ydb__rate__limiter__pb2.AcquireResourceResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Ydb.RateLimiter.V1.RateLimiterService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('Ydb.RateLimiter.V1.RateLimiterService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class RateLimiterService(object):
    """Service that implements distributed rate limiting.

    To use rate limiter functionality you need an existing coordination node.

    Control plane API
    """

    @staticmethod
    def CreateResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.RateLimiter.V1.RateLimiterService/CreateResource',
            protos_dot_ydb__rate__limiter__pb2.CreateResourceRequest.SerializeToString,
            protos_dot_ydb__rate__limiter__pb2.CreateResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def AlterResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.RateLimiter.V1.RateLimiterService/AlterResource',
            protos_dot_ydb__rate__limiter__pb2.AlterResourceRequest.SerializeToString,
            protos_dot_ydb__rate__limiter__pb2.AlterResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DropResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.RateLimiter.V1.RateLimiterService/DropResource',
            protos_dot_ydb__rate__limiter__pb2.DropResourceRequest.SerializeToString,
            protos_dot_ydb__rate__limiter__pb2.DropResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListResources(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.RateLimiter.V1.RateLimiterService/ListResources',
            protos_dot_ydb__rate__limiter__pb2.ListResourcesRequest.SerializeToString,
            protos_dot_ydb__rate__limiter__pb2.ListResourcesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DescribeResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.RateLimiter.V1.RateLimiterService/DescribeResource',
            protos_dot_ydb__rate__limiter__pb2.DescribeResourceRequest.SerializeToString,
            protos_dot_ydb__rate__limiter__pb2.DescribeResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def AcquireResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.RateLimiter.V1.RateLimiterService/AcquireResource',
            protos_dot_ydb__rate__limiter__pb2.AcquireResourceRequest.SerializeToString,
            protos_dot_ydb__rate__limiter__pb2.AcquireResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



from typing import Optional

from .. import issues
from .._topic_common.common import _get_shared_event_loop, CallFromSyncToAsync
from ..aio.coordination.semaphore import CoordinationSemaphore as CoordinationSemaphoreAio
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .session import CoordinationSession


class CoordinationSemaphore:
    def __init__(self, session: "CoordinationSession", name: str, limit: int = 1):
        self._session = session
        self._name = name
        self._limit = limit
        self._closed = False
        self._caller = CallFromSyncToAsync(_get_shared_event_loop())
        self._async_semaphore: CoordinationSemaphoreAio = self._session._async_session.semaphore(name, limit)

    def _check_closed(self):
        if self._closed:
            raise issues.Error(f"CoordinationSemaphore {self._name} already closed")

    def __enter__(self):
        self.acquire()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            self.release()
        except Exception:
            pass

    def acquire(self, count: int = 1, timeout: Optional[float] = None):
        self._check_closed()
        return self._caller.safe_call_with_result(
            self._async_semaphore.acquire(count),
            timeout,
        )

    def release(self, timeout: Optional[float] = None):
        if self._closed:
            return
        return self._caller.safe_call_with_result(
            self._async_semaphore.release(),
            timeout,
        )

    def describe(self, timeout: Optional[float] = None):
        self._check_closed()
        return self._caller.safe_call_with_result(
            self._async_semaphore.describe(),
            timeout,
        )

    def update(self, new_data: bytes, timeout: Optional[float] = None):
        self._check_closed()
        return self._caller.safe_call_with_result(
            self._async_semaphore.update(new_data),
            timeout,
        )

    def close(self, timeout: Optional[float] = None):
        if self._closed:
            return
        try:
            self._caller.safe_call_with_result(
                self._async_semaphore.release(),
                timeout,
            )
        finally:
            self._closed = True



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: ydb_query_v1.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'ydb_query_v1.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v6.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2
from ydb._grpc.v6.protos import ydb_query_pb2 as protos_dot_ydb__query__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12ydb_query_v1.proto\x12\x0cYdb.Query.V1\x1a\x1aprotos/ydb_operation.proto\x1a\x16protos/ydb_query.proto2\xad\x06\n\x0cQueryService\x12R\n\rCreateSession\x12\x1f.Ydb.Query.CreateSessionRequest\x1a .Ydb.Query.CreateSessionResponse\x12R\n\rDeleteSession\x12\x1f.Ydb.Query.DeleteSessionRequest\x1a .Ydb.Query.DeleteSessionResponse\x12K\n\rAttachSession\x12\x1f.Ydb.Query.AttachSessionRequest\x1a\x17.Ydb.Query.SessionState0\x01\x12[\n\x10\x42\x65ginTransaction\x12\".Ydb.Query.BeginTransactionRequest\x1a#.Ydb.Query.BeginTransactionResponse\x12^\n\x11\x43ommitTransaction\x12#.Ydb.Query.CommitTransactionRequest\x1a$.Ydb.Query.CommitTransactionResponse\x12\x64\n\x13RollbackTransaction\x12%.Ydb.Query.RollbackTransactionRequest\x1a&.Ydb.Query.RollbackTransactionResponse\x12U\n\x0c\x45xecuteQuery\x12\x1e.Ydb.Query.ExecuteQueryRequest\x1a#.Ydb.Query.ExecuteQueryResponsePart0\x01\x12K\n\rExecuteScript\x12\x1f.Ydb.Query.ExecuteScriptRequest\x1a\x19.Ydb.Operations.Operation\x12\x61\n\x12\x46\x65tchScriptResults\x12$.Ydb.Query.FetchScriptResultsRequest\x1a%.Ydb.Query.FetchScriptResultsResponseBO\n\x17tech.ydb.proto.query.v1Z4github.com/ydb-platform/ydb-go-genproto/Ydb_Query_V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb_query_v1_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\027tech.ydb.proto.query.v1Z4github.com/ydb-platform/ydb-go-genproto/Ydb_Query_V1'
  _globals['_QUERYSERVICE']._serialized_start=89
  _globals['_QUERYSERVICE']._serialized_end=902
# @@protoc_insertion_point(module_scope)



import pytest

import ydb
from ydb.aio.query.transaction import QueryTxContext
from ydb.query.transaction import QueryTxStateEnum


class TestAsyncQueryTransaction:
    @pytest.mark.asyncio
    async def test_tx_begin(self, tx: QueryTxContext):
        assert tx.tx_id is None

        await tx.begin()
        assert tx.tx_id is not None

    @pytest.mark.asyncio
    async def test_tx_allow_double_commit(self, tx: QueryTxContext):
        await tx.begin()
        await tx.commit()
        await tx.commit()

    @pytest.mark.asyncio
    async def test_tx_allow_double_rollback(self, tx: QueryTxContext):
        await tx.begin()
        await tx.rollback()
        await tx.rollback()

    @pytest.mark.asyncio
    async def test_tx_commit_before_begin(self, tx: QueryTxContext):
        await tx.commit()
        assert tx._tx_state._state == QueryTxStateEnum.COMMITTED

    @pytest.mark.asyncio
    async def test_tx_rollback_before_begin(self, tx: QueryTxContext):
        await tx.rollback()
        assert tx._tx_state._state == QueryTxStateEnum.ROLLBACKED

    @pytest.mark.asyncio
    async def test_tx_first_execute_begins_tx(self, tx: QueryTxContext):
        await tx.execute("select 1;")
        await tx.commit()

    @pytest.mark.asyncio
    async def test_interactive_tx_commit(self, tx: QueryTxContext):
        await tx.execute("select 1;", commit_tx=True)
        with pytest.raises(RuntimeError):
            await tx.execute("select 1;")

    @pytest.mark.asyncio
    async def test_tx_execute_raises_after_commit(self, tx: QueryTxContext):
        await tx.begin()
        await tx.commit()
        with pytest.raises(RuntimeError):
            await tx.execute("select 1;")

    @pytest.mark.asyncio
    async def test_tx_execute_raises_after_rollback(self, tx: QueryTxContext):
        await tx.begin()
        await tx.rollback()
        with pytest.raises(RuntimeError):
            await tx.execute("select 1;")

    @pytest.mark.asyncio
    async def test_context_manager_rollbacks_tx(self, tx: QueryTxContext):
        async with tx:
            await tx.begin()

        assert tx._tx_state._state == QueryTxStateEnum.ROLLBACKED

    @pytest.mark.asyncio
    async def test_context_manager_normal_flow(self, tx: QueryTxContext):
        async with tx:
            await tx.begin()
            await tx.execute("select 1;")
            await tx.commit()

        assert tx._tx_state._state == QueryTxStateEnum.COMMITTED

    @pytest.mark.asyncio
    async def test_context_manager_does_not_hide_exceptions(self, tx: QueryTxContext):
        class CustomException(Exception):
            pass

        with pytest.raises(CustomException):
            async with tx:
                raise CustomException()

    @pytest.mark.asyncio
    async def test_execute_as_context_manager(self, tx: QueryTxContext):
        await tx.begin()

        async with await tx.execute("select 1;") as results:
            res = [result_set async for result_set in results]

        assert len(res) == 1

    @pytest.mark.asyncio
    async def test_execute_two_results(self, tx: QueryTxContext):
        await tx.begin()
        counter = 0
        res = []

        async with await tx.execute("select 1; select 2") as results:
            async for result_set in results:
                counter += 1
                if len(result_set.rows) > 0:
                    res.append(list(result_set.rows[0].values()))

        assert res == [[1], [2]]
        assert counter == 2

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("ydb_terminates_streams_with_unavailable")
    async def test_terminated_stream_raises_ydb_error(self, tx: QueryTxContext):
        await tx.begin()

        with pytest.raises(ydb.Unavailable):
            async with await tx.execute("select 1") as results:
                async for _ in results:
                    pass

    @pytest.mark.asyncio
    async def test_rollback_after_tli_aborted_is_safe(self, pool, table_path: str):
        # Given: a row in a table
        table_ref = f"`{table_path}`"
        await pool.execute_with_retries(f"UPSERT INTO {table_ref} (id, i64Val) VALUES (1, 0);")

        # When: two concurrent transactions try to modify this row.
        async with pool.checkout() as session1, pool.checkout() as session2:
            tx1 = session1.transaction()
            tx2 = session2.transaction()

            async with await tx1.execute(f"SELECT i64Val FROM {table_ref} WHERE id = 1;") as _:
                pass

            async with await tx2.execute(f"UPSERT INTO {table_ref} (id, i64Val) VALUES (1, 1);", commit_tx=True) as _:
                pass

            async with await tx1.execute(f"UPSERT INTO {table_ref} (id, i64Val) VALUES (1, 2);") as _:
                pass
            with pytest.raises(ydb.Aborted):
                await tx1.commit()  # receive TLI here

            # Then: rollback (as a handling of Aborted exception) must be successful.
            await tx1.rollback()



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class IssueMessage(_message.Message):
    __slots__ = ("position", "message", "end_position", "issue_code", "severity", "issues")
    class Position(_message.Message):
        __slots__ = ("row", "column", "file")
        ROW_FIELD_NUMBER: _ClassVar[int]
        COLUMN_FIELD_NUMBER: _ClassVar[int]
        FILE_FIELD_NUMBER: _ClassVar[int]
        row: int
        column: int
        file: str
        def __init__(self, row: _Optional[int] = ..., column: _Optional[int] = ..., file: _Optional[str] = ...) -> None: ...
    POSITION_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    END_POSITION_FIELD_NUMBER: _ClassVar[int]
    ISSUE_CODE_FIELD_NUMBER: _ClassVar[int]
    SEVERITY_FIELD_NUMBER: _ClassVar[int]
    ISSUES_FIELD_NUMBER: _ClassVar[int]
    position: IssueMessage.Position
    message: str
    end_position: IssueMessage.Position
    issue_code: int
    severity: int
    issues: _containers.RepeatedCompositeFieldContainer[IssueMessage]
    def __init__(self, position: _Optional[_Union[IssueMessage.Position, _Mapping]] = ..., message: _Optional[str] = ..., end_position: _Optional[_Union[IssueMessage.Position, _Mapping]] = ..., issue_code: _Optional[int] = ..., severity: _Optional[int] = ..., issues: _Optional[_Iterable[_Union[IssueMessage, _Mapping]]] = ...) -> None: ...



from protos import ydb_operation_pb2 as _ydb_operation_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DatabaseInfo(_message.Message):
    __slots__ = ["endpoint", "id", "location", "name", "path", "status", "weight"]
    class Status(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    AVAILABLE: DatabaseInfo.Status
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    READ_ONLY: DatabaseInfo.Status
    STATUS_FIELD_NUMBER: _ClassVar[int]
    STATUS_UNSPECIFIED: DatabaseInfo.Status
    UNAVAILABLE: DatabaseInfo.Status
    WEIGHT_FIELD_NUMBER: _ClassVar[int]
    endpoint: str
    id: str
    location: str
    name: str
    path: str
    status: DatabaseInfo.Status
    weight: int
    def __init__(self, name: _Optional[str] = ..., path: _Optional[str] = ..., id: _Optional[str] = ..., endpoint: _Optional[str] = ..., location: _Optional[str] = ..., status: _Optional[_Union[DatabaseInfo.Status, str]] = ..., weight: _Optional[int] = ...) -> None: ...

class ListFederationDatabasesRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListFederationDatabasesResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ListFederationDatabasesResult(_message.Message):
    __slots__ = ["control_plane_endpoint", "federation_databases", "self_location"]
    CONTROL_PLANE_ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    FEDERATION_DATABASES_FIELD_NUMBER: _ClassVar[int]
    SELF_LOCATION_FIELD_NUMBER: _ClassVar[int]
    control_plane_endpoint: str
    federation_databases: _containers.RepeatedCompositeFieldContainer[DatabaseInfo]
    self_location: str
    def __init__(self, control_plane_endpoint: _Optional[str] = ..., federation_databases: _Optional[_Iterable[_Union[DatabaseInfo, _Mapping]]] = ..., self_location: _Optional[str] = ...) -> None: ...



import logging
from typing import (
    Optional,
    TYPE_CHECKING,
)

from .base import AsyncResponseContextIterator
from ... import issues
from ...settings import BaseRequestSettings
from ...query import base
from ...query.transaction import (
    BaseQueryTxContext,
    QueryTxStateEnum,
)

if TYPE_CHECKING:
    from .session import QuerySession

logger = logging.getLogger(__name__)


class QueryTxContext(BaseQueryTxContext):
    def __init__(self, driver, session: "QuerySession", tx_mode: base.BaseQueryTxMode):
        """
        An object that provides a simple transaction context manager that allows statements execution
        in a transaction. You don't have to open transaction explicitly, because context manager encapsulates
        transaction control logic, and opens new transaction if:

        1) By explicit .begin() method;
        2) On execution of a first statement, which is strictly recommended method, because that avoids useless round trip

        This context manager is not thread-safe, so you should not manipulate on it concurrently.

        :param driver: A driver instance
        :param session: A session instance
        :param tx_mode: Transaction mode, which is a one from the following choices:
         1) QuerySerializableReadWrite() which is default mode;
         2) QueryOnlineReadOnly(allow_inconsistent_reads=False);
         3) QuerySnapshotReadOnly();
         4) QuerySnapshotReadWrite();
         5) QueryStaleReadOnly().
        """
        super().__init__(driver, session, tx_mode)
        self._init_callback_handler(base.CallbackHandlerMode.ASYNC)

    async def __aenter__(self) -> "QueryTxContext":
        """
        Enters a context manager and returns a transaction

        :return: A transaction instance
        """
        return self

    async def __aexit__(self, *args, **kwargs):
        """
        Closes a transaction context manager and rollbacks transaction if
        it is not finished explicitly
        """
        await self._ensure_prev_stream_finished()
        if self._tx_state._state == QueryTxStateEnum.BEGINED and self._external_error is None:
            # It's strictly recommended to close transactions directly
            # by using commit_tx=True flag while executing statement or by
            # .commit() or .rollback() methods, but here we trying to do best
            # effort to avoid useless open transactions
            logger.warning("Potentially leaked tx: %s", self._tx_state.tx_id)
            try:
                await self.rollback()
            except issues.Error:
                logger.warning("Failed to rollback leaked tx: %s", self._tx_state.tx_id)

    async def _ensure_prev_stream_finished(self) -> None:
        if self._prev_stream is not None:
            async with self._prev_stream:
                pass
            self._prev_stream = None

    async def begin(self, settings: Optional[BaseRequestSettings] = None) -> "QueryTxContext":
        """Explicitly begins a transaction

        :param settings: An additional request settings BaseRequestSettings;

        :return: None or exception if begin is failed
        """
        await self._begin_call(settings)
        return self

    async def commit(self, settings: Optional[BaseRequestSettings] = None) -> None:
        """Calls commit on a transaction if it is open otherwise is no-op. If transaction execution
        failed then this method raises PreconditionFailed.

        :param settings: An additional request settings BaseRequestSettings;

        :return: A committed transaction or exception if commit is failed
        """
        self._check_external_error_set()

        if self._tx_state._should_skip(QueryTxStateEnum.COMMITTED):
            return

        if self._tx_state._state == QueryTxStateEnum.NOT_INITIALIZED:
            self._tx_state._change_state(QueryTxStateEnum.COMMITTED)
            return

        await self._ensure_prev_stream_finished()

        try:
            await self._execute_callbacks_async(base.TxEvent.BEFORE_COMMIT)
            await self._commit_call(settings)
            await self._execute_callbacks_async(base.TxEvent.AFTER_COMMIT, exc=None)
        except BaseException as e:
            await self._execute_callbacks_async(base.TxEvent.AFTER_COMMIT, exc=e)
            raise e

    async def rollback(self, settings: Optional[BaseRequestSettings] = None) -> None:
        """Calls rollback on a transaction if it is open otherwise is no-op. If transaction execution
        failed then this method raises PreconditionFailed.

        :param settings: An additional request settings BaseRequestSettings;

        :return: A committed transaction or exception if commit is failed
        """
        self._check_external_error_set()

        if self._tx_state._should_skip(QueryTxStateEnum.ROLLBACKED):
            return

        if self._tx_state._state == QueryTxStateEnum.NOT_INITIALIZED:
            self._tx_state._change_state(QueryTxStateEnum.ROLLBACKED)
            return

        await self._ensure_prev_stream_finished()

        try:
            await self._execute_callbacks_async(base.TxEvent.BEFORE_ROLLBACK)
            await self._rollback_call(settings)
            await self._execute_callbacks_async(base.TxEvent.AFTER_ROLLBACK, exc=None)
        except BaseException as e:
            await self._execute_callbacks_async(base.TxEvent.AFTER_ROLLBACK, exc=e)
            raise e

    async def execute(
        self,
        query: str,
        parameters: Optional[dict] = None,
        commit_tx: Optional[bool] = False,
        syntax: Optional[base.QuerySyntax] = None,
        exec_mode: Optional[base.QueryExecMode] = None,
        concurrent_result_sets: Optional[bool] = False,
        settings: Optional[BaseRequestSettings] = None,
        *,
        stats_mode: Optional[base.QueryStatsMode] = None,
        schema_inclusion_mode: Optional[base.QuerySchemaInclusionMode] = None,
        result_set_format: Optional[base.QueryResultSetFormat] = None,
        arrow_format_settings: Optional[base.ArrowFormatSettings] = None,
    ) -> AsyncResponseContextIterator:
        """Sends a query to Query Service

        :param query: (YQL or SQL text) to be executed.
        :param parameters: dict with parameters and YDB types;
        :param commit_tx: A special flag that allows transaction commit.
        :param syntax: Syntax of the query, which is a one from the following choices:
         1) QuerySyntax.YQL_V1, which is default;
         2) QuerySyntax.PG.
        :param exec_mode: Exec mode of the query, which is a one from the following choices:
         1) QueryExecMode.EXECUTE, which is default;
         2) QueryExecMode.EXPLAIN;
         3) QueryExecMode.VALIDATE;
         4) QueryExecMode.PARSE.
        :param concurrent_result_sets: A flag to allow YDB mix parts of different result sets. Default is False;
        :param stats_mode: Mode of query statistics to gather, which is a one from the following choices:
         1) QueryStatsMode:NONE, which is default;
         2) QueryStatsMode.BASIC;
         3) QueryStatsMode.FULL;
         4) QueryStatsMode.PROFILE;
        :param schema_inclusion_mode: Schema inclusion mode for result sets:
         1) QuerySchemaInclusionMode.ALWAYS, which is default;
         2) QuerySchemaInclusionMode.FIRST_ONLY.
        :param result_set_format: Format of the result sets:
         1) QueryResultSetFormat.VALUE, which is default;
         2) QueryResultSetFormat.ARROW.
        :param arrow_format_settings: Settings for Arrow format when result_set_format is ARROW.

        :return: Iterator with result sets
        """
        await self._ensure_prev_stream_finished()

        stream_it = await self._execute_call(
            query=query,
            parameters=parameters,
            commit_tx=commit_tx,
            syntax=syntax,
            exec_mode=exec_mode,
            stats_mode=stats_mode,
            schema_inclusion_mode=schema_inclusion_mode,
            result_set_format=result_set_format,
            arrow_format_settings=arrow_format_settings,
            concurrent_result_sets=concurrent_result_sets,
            settings=settings,
        )

        self._prev_stream = AsyncResponseContextIterator(
            it=stream_it,
            wrapper=lambda resp: base.wrap_execute_query_response(
                rpc_state=None,
                response_pb=resp,
                session=self.session,
                tx=self,
                commit_tx=commit_tx,
                settings=self.session._settings,
            ),
            on_error=self.session._on_execute_stream_error,
        )
        return self._prev_stream



from . import connection as conn_impl

from ydb import _apis, settings as settings_impl
from ydb.resolver import (
    DiscoveryResult,
    DiscoveryEndpointsResolver as _DiscoveryEndpointsResolver,
    _list_endpoints_request_factory,
)


class _FakeLock:
    def __init__(self):
        pass

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass


class DiscoveryEndpointsResolver(_DiscoveryEndpointsResolver):
    def __init__(self, driver_config):
        super().__init__(driver_config)
        self._lock = _FakeLock()

    async def resolve(self):
        self.logger.debug("Preparing initial endpoint to resolve endpoints")
        endpoint = next(self._endpoints_iter)
        connection = conn_impl.Connection(endpoint, self._driver_config)
        try:
            await connection.connection_ready()
        except BaseException:
            self._add_debug_details(
                'Failed to establish connection to YDB discovery endpoint: "%s". Check endpoint correctness.' % endpoint
            )
            return None
        self.logger.debug("Resolving endpoints for database %s", self._driver_config.database)

        try:
            resolved = await connection(
                _list_endpoints_request_factory(self._driver_config),
                _apis.DiscoveryService.Stub,
                _apis.DiscoveryService.ListEndpoints,
                DiscoveryResult.from_response,
                settings=settings_impl.BaseRequestSettings().with_timeout(self._ready_timeout),
            )

            self._add_debug_details(
                "Resolved endpoints for database %s: %s",
                self._driver_config.database,
                resolved,
            )

            return resolved
        except BaseException as e:

            self._add_debug_details(
                'Failed to resolve endpoints for database %s. Endpoint: "%s". Error details:\n %s',
                self._driver_config.database,
                endpoint,
                e,
            )

        finally:
            await connection.close()

        return None



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class StatusIds(_message.Message):
    __slots__ = []
    class StatusCode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    ABORTED: StatusIds.StatusCode
    ALREADY_EXISTS: StatusIds.StatusCode
    BAD_REQUEST: StatusIds.StatusCode
    BAD_SESSION: StatusIds.StatusCode
    CANCELLED: StatusIds.StatusCode
    EXTERNAL_ERROR: StatusIds.StatusCode
    GENERIC_ERROR: StatusIds.StatusCode
    INTERNAL_ERROR: StatusIds.StatusCode
    NOT_FOUND: StatusIds.StatusCode
    OVERLOADED: StatusIds.StatusCode
    PRECONDITION_FAILED: StatusIds.StatusCode
    SCHEME_ERROR: StatusIds.StatusCode
    SESSION_BUSY: StatusIds.StatusCode
    SESSION_EXPIRED: StatusIds.StatusCode
    STATUS_CODE_UNSPECIFIED: StatusIds.StatusCode
    SUCCESS: StatusIds.StatusCode
    TIMEOUT: StatusIds.StatusCode
    UNAUTHORIZED: StatusIds.StatusCode
    UNAVAILABLE: StatusIds.StatusCode
    UNDETERMINED: StatusIds.StatusCode
    UNSUPPORTED: StatusIds.StatusCode
    def __init__(self) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from ydb._grpc.v6.draft.protos import ydb_federated_query_pb2 as draft_dot_protos_dot_ydb__federated__query__pb2

GRPC_GENERATED_VERSION = '1.76.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in draft/ydb_federated_query_v1_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class FederatedQueryServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateQuery = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/CreateQuery',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateQueryRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateQueryResponse.FromString,
                _registered_method=True)
        self.ListQueries = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/ListQueries',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListQueriesRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListQueriesResponse.FromString,
                _registered_method=True)
        self.DescribeQuery = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/DescribeQuery',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeQueryRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeQueryResponse.FromString,
                _registered_method=True)
        self.GetQueryStatus = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/GetQueryStatus',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.GetQueryStatusRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.GetQueryStatusResponse.FromString,
                _registered_method=True)
        self.ModifyQuery = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/ModifyQuery',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyQueryRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyQueryResponse.FromString,
                _registered_method=True)
        self.DeleteQuery = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/DeleteQuery',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteQueryRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteQueryResponse.FromString,
                _registered_method=True)
        self.ControlQuery = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/ControlQuery',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ControlQueryRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ControlQueryResponse.FromString,
                _registered_method=True)
        self.GetResultData = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/GetResultData',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.GetResultDataRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.GetResultDataResponse.FromString,
                _registered_method=True)
        self.ListJobs = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/ListJobs',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListJobsRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListJobsResponse.FromString,
                _registered_method=True)
        self.DescribeJob = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/DescribeJob',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeJobRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeJobResponse.FromString,
                _registered_method=True)
        self.CreateConnection = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/CreateConnection',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateConnectionRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateConnectionResponse.FromString,
                _registered_method=True)
        self.ListConnections = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/ListConnections',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListConnectionsRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListConnectionsResponse.FromString,
                _registered_method=True)
        self.DescribeConnection = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/DescribeConnection',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeConnectionRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeConnectionResponse.FromString,
                _registered_method=True)
        self.ModifyConnection = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/ModifyConnection',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyConnectionRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyConnectionResponse.FromString,
                _registered_method=True)
        self.DeleteConnection = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/DeleteConnection',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteConnectionRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteConnectionResponse.FromString,
                _registered_method=True)
        self.TestConnection = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/TestConnection',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.TestConnectionRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.TestConnectionResponse.FromString,
                _registered_method=True)
        self.CreateBinding = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/CreateBinding',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateBindingRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateBindingResponse.FromString,
                _registered_method=True)
        self.ListBindings = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/ListBindings',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListBindingsRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListBindingsResponse.FromString,
                _registered_method=True)
        self.DescribeBinding = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/DescribeBinding',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeBindingRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeBindingResponse.FromString,
                _registered_method=True)
        self.ModifyBinding = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/ModifyBinding',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyBindingRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyBindingResponse.FromString,
                _registered_method=True)
        self.DeleteBinding = channel.unary_unary(
                '/FederatedQuery.V1.FederatedQueryService/DeleteBinding',
                request_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteBindingRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteBindingResponse.FromString,
                _registered_method=True)


class FederatedQueryServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateQuery(self, request, context):
        """Query
        Query is the text of an SQL request, the results of the last run and the state after the last run (partitions offsets, consumer in YDS)
        Create a query object with a given SQL
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListQueries(self, request, context):
        """Get a list of brief queries objects
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeQuery(self, request, context):
        """Get full information about the object of the query
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetQueryStatus(self, request, context):
        """Get status of the query
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModifyQuery(self, request, context):
        """Change the attributes of the query (acl, name, ...)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteQuery(self, request, context):
        """Completely delete the query
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ControlQuery(self, request, context):
        """Change the state of the query lifecycle
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetResultData(self, request, context):
        """Get a results page
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListJobs(self, request, context):
        """Job
        Job - appears immediately after starting the request and contains the request metadata
        Get a list of jobs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeJob(self, request, context):
        """Get information about the job
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateConnection(self, request, context):
        """Connection
        Connection - entity that describes connection points. This can be imagined as an analogue of a network address.
        Create a connection object (ObjectStorage, YDB, YDS, ...)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListConnections(self, request, context):
        """Get a list of connections objects
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeConnection(self, request, context):
        """Get information about the object of the connection
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModifyConnection(self, request, context):
        """Change the attributes of the connection
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteConnection(self, request, context):
        """Completely delete the connection
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TestConnection(self, request, context):
        """Test the connection (permissions, network, ...)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateBinding(self, request, context):
        """Binding
        Binding - entity using which a schema is assigned to non-schematic data
        Create a binding object - bind schema with ObjectStorage object or YDS stream
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListBindings(self, request, context):
        """Get a list of bindings objects
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeBinding(self, request, context):
        """Get information about the object of the binding
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModifyBinding(self, request, context):
        """Change the attributes of the binding
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteBinding(self, request, context):
        """Completely delete the binding
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_FederatedQueryServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateQuery,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateQueryRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateQueryResponse.SerializeToString,
            ),
            'ListQueries': grpc.unary_unary_rpc_method_handler(
                    servicer.ListQueries,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListQueriesRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListQueriesResponse.SerializeToString,
            ),
            'DescribeQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeQuery,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeQueryRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeQueryResponse.SerializeToString,
            ),
            'GetQueryStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetQueryStatus,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.GetQueryStatusRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.GetQueryStatusResponse.SerializeToString,
            ),
            'ModifyQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.ModifyQuery,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyQueryRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyQueryResponse.SerializeToString,
            ),
            'DeleteQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteQuery,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteQueryRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteQueryResponse.SerializeToString,
            ),
            'ControlQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.ControlQuery,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ControlQueryRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ControlQueryResponse.SerializeToString,
            ),
            'GetResultData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetResultData,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.GetResultDataRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.GetResultDataResponse.SerializeToString,
            ),
            'ListJobs': grpc.unary_unary_rpc_method_handler(
                    servicer.ListJobs,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListJobsRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListJobsResponse.SerializeToString,
            ),
            'DescribeJob': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeJob,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeJobRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeJobResponse.SerializeToString,
            ),
            'CreateConnection': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateConnection,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateConnectionRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateConnectionResponse.SerializeToString,
            ),
            'ListConnections': grpc.unary_unary_rpc_method_handler(
                    servicer.ListConnections,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListConnectionsRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListConnectionsResponse.SerializeToString,
            ),
            'DescribeConnection': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeConnection,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeConnectionRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeConnectionResponse.SerializeToString,
            ),
            'ModifyConnection': grpc.unary_unary_rpc_method_handler(
                    servicer.ModifyConnection,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyConnectionRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyConnectionResponse.SerializeToString,
            ),
            'DeleteConnection': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteConnection,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteConnectionRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteConnectionResponse.SerializeToString,
            ),
            'TestConnection': grpc.unary_unary_rpc_method_handler(
                    servicer.TestConnection,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.TestConnectionRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.TestConnectionResponse.SerializeToString,
            ),
            'CreateBinding': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateBinding,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateBindingRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.CreateBindingResponse.SerializeToString,
            ),
            'ListBindings': grpc.unary_unary_rpc_method_handler(
                    servicer.ListBindings,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListBindingsRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ListBindingsResponse.SerializeToString,
            ),
            'DescribeBinding': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeBinding,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeBindingRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DescribeBindingResponse.SerializeToString,
            ),
            'ModifyBinding': grpc.unary_unary_rpc_method_handler(
                    servicer.ModifyBinding,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyBindingRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.ModifyBindingResponse.SerializeToString,
            ),
            'DeleteBinding': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteBinding,
                    request_deserializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteBindingRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__federated__query__pb2.DeleteBindingResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'FederatedQuery.V1.FederatedQueryService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('FederatedQuery.V1.FederatedQueryService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class FederatedQueryService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/CreateQuery',
            draft_dot_protos_dot_ydb__federated__query__pb2.CreateQueryRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.CreateQueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListQueries(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/ListQueries',
            draft_dot_protos_dot_ydb__federated__query__pb2.ListQueriesRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.ListQueriesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DescribeQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/DescribeQuery',
            draft_dot_protos_dot_ydb__federated__query__pb2.DescribeQueryRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.DescribeQueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetQueryStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/GetQueryStatus',
            draft_dot_protos_dot_ydb__federated__query__pb2.GetQueryStatusRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.GetQueryStatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ModifyQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/ModifyQuery',
            draft_dot_protos_dot_ydb__federated__query__pb2.ModifyQueryRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.ModifyQueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/DeleteQuery',
            draft_dot_protos_dot_ydb__federated__query__pb2.DeleteQueryRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.DeleteQueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ControlQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/ControlQuery',
            draft_dot_protos_dot_ydb__federated__query__pb2.ControlQueryRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.ControlQueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetResultData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/GetResultData',
            draft_dot_protos_dot_ydb__federated__query__pb2.GetResultDataRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.GetResultDataResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListJobs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/ListJobs',
            draft_dot_protos_dot_ydb__federated__query__pb2.ListJobsRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.ListJobsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DescribeJob(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/DescribeJob',
            draft_dot_protos_dot_ydb__federated__query__pb2.DescribeJobRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.DescribeJobResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreateConnection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/CreateConnection',
            draft_dot_protos_dot_ydb__federated__query__pb2.CreateConnectionRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.CreateConnectionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListConnections(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/ListConnections',
            draft_dot_protos_dot_ydb__federated__query__pb2.ListConnectionsRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.ListConnectionsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DescribeConnection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/DescribeConnection',
            draft_dot_protos_dot_ydb__federated__query__pb2.DescribeConnectionRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.DescribeConnectionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ModifyConnection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/ModifyConnection',
            draft_dot_protos_dot_ydb__federated__query__pb2.ModifyConnectionRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.ModifyConnectionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteConnection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/DeleteConnection',
            draft_dot_protos_dot_ydb__federated__query__pb2.DeleteConnectionRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.DeleteConnectionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def TestConnection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/TestConnection',
            draft_dot_protos_dot_ydb__federated__query__pb2.TestConnectionRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.TestConnectionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreateBinding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/CreateBinding',
            draft_dot_protos_dot_ydb__federated__query__pb2.CreateBindingRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.CreateBindingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListBindings(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/ListBindings',
            draft_dot_protos_dot_ydb__federated__query__pb2.ListBindingsRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.ListBindingsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DescribeBinding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/DescribeBinding',
            draft_dot_protos_dot_ydb__federated__query__pb2.DescribeBindingRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.DescribeBindingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ModifyBinding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/ModifyBinding',
            draft_dot_protos_dot_ydb__federated__query__pb2.ModifyBindingRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.ModifyBindingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteBinding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/FederatedQuery.V1.FederatedQueryService/DeleteBinding',
            draft_dot_protos_dot_ydb__federated__query__pb2.DeleteBindingRequest.SerializeToString,
            draft_dot_protos_dot_ydb__federated__query__pb2.DeleteBindingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import ydb
import pytest

from os import path


def test_scheme_error(driver_sync, database):
    session = driver_sync.table_client.session().create()
    with pytest.raises(ydb.issues.SchemeError) as exc:
        session.drop_table(path.join(database, "foobartable"))

    server_code = ydb.issues.StatusCode.SCHEME_ERROR

    assert type(exc.value) == ydb.issues.SchemeError
    assert exc.value.status == server_code
    assert f"server_code: {server_code}" in str(exc.value)
    assert "Path does not exist" in str(exc.value)



# -*- coding: utf-8 -*-
import abc
import enum
from abc import abstractmethod
from . import issues, operation, settings as settings_impl, _apis


@enum.unique
class SchemeEntryType(enum.IntEnum):
    """
    Enumerates all available entry types.
    """

    TYPE_UNSPECIFIED = 0
    DIRECTORY = 1
    TABLE = 2
    PERS_QUEUE_GROUP = 3
    DATABASE = 4
    RTMR_VOLUME = 5
    BLOCK_STORE_VOLUME = 6
    COORDINATION_NODE = 7
    COLUMN_STORE = 12
    COLUMN_TABLE = 13
    SEQUENCE = 15
    REPLICATION = 16
    TOPIC = 17
    EXTERNAL_TABLE = 18
    EXTERNAL_DATA_SOURCE = 19
    VIEW = 20
    RESOURCE_POOL = 21
    TRANSFER = 23
    SYS_VIEW = 24

    @classmethod
    def _missing_(cls, value):
        return cls.TYPE_UNSPECIFIED

    @staticmethod
    def is_table(entry):
        """
        Deprecated, use is_row_table instead of this.

        :param entry: A scheme entry to check
        :return: True if scheme entry is a row table and False otherwise  (same as is_row_table)
        """
        return entry == SchemeEntryType.TABLE

    @staticmethod
    def is_any_table(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is table (independent of table type) and False otherwise
        """
        return entry in (SchemeEntryType.TABLE, SchemeEntryType.COLUMN_TABLE)

    @staticmethod
    def is_column_table(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a column table and False otherwise
        """
        return entry == SchemeEntryType.COLUMN_TABLE

    @staticmethod
    def is_column_store(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a column store and False otherwise
        """
        return entry == SchemeEntryType.COLUMN_STORE

    @staticmethod
    def is_row_table(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a row table and False otherwise (same as is_table)
        """
        return entry == SchemeEntryType.TABLE

    @staticmethod
    def is_directory(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a directory and False otherwise
        """
        return entry == SchemeEntryType.DIRECTORY

    @staticmethod
    def is_database(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a database and False otherwise
        """
        return entry == SchemeEntryType.DATABASE

    @staticmethod
    def is_coordination_node(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a coordination node and False otherwise
        """
        return entry == SchemeEntryType.COORDINATION_NODE

    @staticmethod
    def is_directory_or_database(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a directory or database and False otherwise
        """
        return entry == SchemeEntryType.DATABASE or entry == SchemeEntryType.DIRECTORY

    @staticmethod
    def is_external_table(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is an external table and False otherwise
        """
        return entry == SchemeEntryType.EXTERNAL_TABLE

    @staticmethod
    def is_external_data_source(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is an external data source and False otherwise
        """
        return entry == SchemeEntryType.EXTERNAL_DATA_SOURCE

    @staticmethod
    def is_view(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a view and False otherwise
        """
        return entry == SchemeEntryType.VIEW

    @staticmethod
    def is_resource_pool(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a resource pool and False otherwise
        """
        return entry == SchemeEntryType.RESOURCE_POOL

    @staticmethod
    def is_topic(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a topic and False otherwise
        """
        return entry == SchemeEntryType.TOPIC

    @staticmethod
    def is_sysview(entry):
        """
        :param entry: A scheme entry to check
        :return: True if scheme entry is a system view and False otherwise
        """
        return entry == SchemeEntryType.SYS_VIEW


class SchemeEntry(object):
    __slots__ = (
        "name",
        "owner",
        "type",
        "effective_permissions",
        "permissions",
        "size_bytes",
    )

    def __init__(self, name, owner, type, effective_permissions, permissions, size_bytes, *args, **kwargs):
        """
        Represents a scheme entry.
        :param name: A name of a scheme entry
        :param owner: A owner of a scheme entry
        :param type: A type of scheme entry
        :param effective_permissions: A list of effective permissions applied to this scheme entry
        :param permissions: A list of permissions applied to this scheme entry
        :param size_bytes: Size of entry in bytes
        """
        self.name = name
        self.owner = owner
        self.type = type
        self.effective_permissions = effective_permissions
        self.permissions = permissions
        self.size_bytes = size_bytes

    def is_directory(self):
        """
        :return: True if scheme entry is a directory and False otherwise
        """
        return SchemeEntryType.is_directory(self.type)

    def is_column_store(self):
        """
        :return: True if scheme entry is a column store and False otherwise
        """
        return SchemeEntryType.is_column_store(self.type)

    def is_table(self):
        """
        :return: True if scheme entry is a row table and False otherwise (same as is_row_table)
        """
        return SchemeEntryType.is_table(self.type)

    def is_column_table(self):
        """
        :return: True if scheme entry is a column table and False otherwise (same as is_row_table)
        """
        return SchemeEntryType.is_column_table(self.type)

    def is_row_table(self):
        """
        :return: True if scheme entry is a row table and False otherwise (same as is_table)
        """
        return SchemeEntryType.is_table(self.type)

    def is_any_table(self):
        """
        :return: True if scheme entry is table (independent of table type) and False otherwise
        """
        return SchemeEntryType.is_any_table(self.type)

    def is_database(self):
        """
        :return: True if scheme entry is a database and False otherwise
        """
        return SchemeEntryType.is_database(self.type)

    def is_directory_or_database(self):
        """
        :return: True if scheme entry is a directory or a database and False otherwise
        """
        return SchemeEntryType.is_directory_or_database(self.type)

    def is_coordination_node(self):
        """
        :return: True if scheme entry is a coordination node and False otherwise
        """
        return SchemeEntryType.is_coordination_node(self.type)

    def is_external_table(self):
        """
        :return: True if scheme entry is an external table and False otherwise
        """
        return SchemeEntryType.is_external_table(self.type)

    def is_external_data_source(self):
        """
        :return: True if scheme entry is an external data source and False otherwise
        """
        return SchemeEntryType.is_external_data_source(self.type)

    def is_view(self):
        """
        :return: True if scheme entry is a view and False otherwise
        """
        return SchemeEntryType.is_view(self.type)

    def is_resource_pool(self):
        """
        :return: True if scheme entry is a resource pool and False otherwise
        """
        return SchemeEntryType.is_resource_pool(self.type)

    def is_sysview(self):
        """
        :return: True if scheme entry is a system view and False otherwise
        """
        return SchemeEntryType.is_sysview(self.type)


class Directory(SchemeEntry):
    __slots__ = ("children",)

    def __init__(self, name, owner, type, effective_permissions, permissions, children, *args, **kwargs):
        """
        Represents a directory scheme entry.
        :param name: A name of a scheme entry
        :param owner: A owner of a scheme entry
        :param type: A type of scheme entry
        :param effective_permissions: A list of effective permissions applied to this scheme entry
        :param permissions: A list of permissions applied to this scheme entry
        :param children: A list of children
        """
        super(Directory, self).__init__(name, owner, type, effective_permissions, permissions, 0)
        self.children = children


def _describe_path_request_factory(path):
    request = _apis.ydb_scheme.DescribePathRequest()
    request.path = path
    return request


def _list_directory_request_factory(path):
    request = _apis.ydb_scheme.ListDirectoryRequest()
    request.path = path
    return request


def _remove_directory_request_factory(path):
    request = _apis.ydb_scheme.RemoveDirectoryRequest()
    request.path = path
    return request


def _make_directory_request_factory(path):
    request = _apis.ydb_scheme.MakeDirectoryRequest()
    request.path = path
    return request


class MakeDirectorySettings(settings_impl.BaseRequestSettings):
    pass


class RemoveDirectorySettings(settings_impl.BaseRequestSettings):
    pass


class ListDirectorySettings(settings_impl.BaseRequestSettings):
    pass


class DescribePathSettings(settings_impl.BaseRequestSettings):
    pass


class ModifyPermissionsSettings(settings_impl.BaseRequestSettings):
    def __init__(self):
        super(ModifyPermissionsSettings, self).__init__()
        self._pb = _apis.ydb_scheme.ModifyPermissionsRequest()

    def grant_permissions(self, subject, permission_names):
        permission_action = self._pb.actions.add()
        permission_action.grant.MergeFrom(Permissions(subject, permission_names).to_pb())
        return self

    def revoke_permissions(self, subject, permission_names):
        permission_action = self._pb.actions.add()
        permission_action.revoke.MergeFrom(Permissions(subject, permission_names).to_pb())
        return self

    def set_permissions(self, subject, permission_names):
        permission_action = self._pb.actions.add()
        permission_action.set.MergeFrom(Permissions(subject, permission_names).to_pb())
        return self

    def change_owner(self, owner):
        permission_action = self._pb.actions.add()
        permission_action.change_owner = owner
        return self

    def clear_permissions(self):
        self._pb.clear_permissions = True
        return self

    def to_pb(self):
        return self._pb


class Permissions(object):
    __slots__ = ("subject", "permission_names")

    def __init__(self, subject, permission_names):
        """
        Represents permissions
        :param subject: A subject of permission names
        :param permission_names: A list of permission names
        """
        self.subject = subject
        self.permission_names = permission_names

    def to_pb(self):
        """
        :return: A protocol buffer representation of permissions
        """
        pb = _apis.ydb_scheme.Permissions()
        pb.subject = self.subject
        pb.permission_names.extend(self.permission_names)
        return pb


def _modify_permissions_request_factory(path, settings):
    """
    Constructs modify permissions request
    :param path: A path to apply permissions
    :param settings: An instance of ModifyPermissionsSettings
    :return: A constructed request
    """
    modify_permissions_request = settings.to_pb()
    modify_permissions_request.path = path
    return modify_permissions_request


def _wrap_permissions(permissions):
    """
    Wraps permissions protocol buffers into native Python objects
    :param permissions: A protocol buffer representation of permissions
    :return: A iterable of permissions
    """
    return tuple(Permissions(permission.subject, permission.permission_names) for permission in permissions)


def _wrap_scheme_entry(entry_pb, scheme_entry_cls=None, *args, **kwargs):
    """
    Wraps scheme entry into native Python objects.
    :param entry_pb: A protocol buffer representation of a scheme entry
    :param scheme_entry_cls: A native Python class that represents scheme entry (
    by default that is generic SchemeEntry)
    :param args: A list of optional arguments
    :param kwargs: A dictionary of with optional arguments
    :return: A native Python reprensentation of scheme entry
    """
    scheme_entry_cls = SchemeEntry if scheme_entry_cls is None else scheme_entry_cls
    return scheme_entry_cls(
        entry_pb.name,
        entry_pb.owner,
        SchemeEntryType(entry_pb.type),
        _wrap_permissions(entry_pb.effective_permissions),
        _wrap_permissions(entry_pb.permissions),
        entry_pb.size_bytes,
        *args,
        **kwargs
    )


def _wrap_list_directory_response(rpc_state, response):
    """
    Wraps list directory response
    :param response: A list directory response
    :return: A directory
    """
    issues._process_response(response.operation)
    message = _apis.ydb_scheme.ListDirectoryResult()
    response.operation.result.Unpack(message)
    children = []
    supported_items = set(i.value for i in SchemeEntryType)
    for children_item in message.children:
        if children_item.type not in supported_items:
            continue

        children.append(_wrap_scheme_entry(children_item))

    return Directory(
        message.self.name,
        message.self.owner,
        SchemeEntryType(message.self.type),
        _wrap_permissions(message.self.effective_permissions),
        _wrap_permissions(message.self.permissions),
        tuple(children),
    )


def _wrap_describe_path_response(rpc_state, response):
    issues._process_response(response.operation)
    message = _apis.ydb_scheme.DescribePathResult()
    response.operation.result.Unpack(message)
    return _wrap_scheme_entry(message.self)


class ISchemeClient(abc.ABC):
    @abstractmethod
    def __init__(self, driver):
        pass

    @abstractmethod
    def make_directory(self, path, settings):
        pass

    @abstractmethod
    def remove_directory(self, path, settings):
        pass

    @abstractmethod
    def list_directory(self, path, settings):
        pass

    @abstractmethod
    def describe_path(self, path, settings):
        pass

    @abstractmethod
    def modify_permissions(self, path, settings):
        """
        Modifies permissions for provided scheme entry

        :param path: A path of scheme entry
        :param settings: An instance of ModifyPermissionsSettings

        :return: An operation if success or exception on case of failure
        """
        pass


class BaseSchemeClient(ISchemeClient):
    __slots__ = ("_driver",)

    def __init__(self, driver):
        self._driver = driver

    def make_directory(self, path, settings=None):
        return self._driver(
            _make_directory_request_factory(path),
            _apis.SchemeService.Stub,
            _apis.SchemeService.MakeDirectory,
            operation.Operation,
            settings,
        )

    def remove_directory(self, path, settings=None):
        return self._driver(
            _remove_directory_request_factory(path),
            _apis.SchemeService.Stub,
            _apis.SchemeService.RemoveDirectory,
            operation.Operation,
            settings,
        )

    def list_directory(self, path, settings=None):
        return self._driver(
            _list_directory_request_factory(path),
            _apis.SchemeService.Stub,
            _apis.SchemeService.ListDirectory,
            _wrap_list_directory_response,
            settings,
        )

    def describe_path(self, path, settings=None):
        return self._driver(
            _describe_path_request_factory(path),
            _apis.SchemeService.Stub,
            _apis.SchemeService.DescribePath,
            _wrap_describe_path_response,
            settings,
        )

    def modify_permissions(self, path, settings):
        """
        Modifies permissions for provided scheme entry

        :param path: A path of scheme entry
        :param settings: An instance of ModifyPermissionsSettings

        :return: An operation if success or exception on case of failure
        """
        return self._driver(
            _modify_permissions_request_factory(path, settings),
            _apis.SchemeService.Stub,
            _apis.SchemeService.ModifyPermissions,
            operation.Operation,
            settings,
        )


class SchemeClient(BaseSchemeClient):
    def async_make_directory(self, path, settings=None):
        return self._driver.future(
            _make_directory_request_factory(path),
            _apis.SchemeService.Stub,
            _apis.SchemeService.MakeDirectory,
            operation.Operation,
            settings,
        )

    def async_remove_directory(self, path, settings=None):
        return self._driver.future(
            _remove_directory_request_factory(path),
            _apis.SchemeService.Stub,
            _apis.SchemeService.RemoveDirectory,
            operation.Operation,
            settings,
        )

    def async_list_directory(self, path, settings=None):
        return self._driver.future(
            _list_directory_request_factory(path),
            _apis.SchemeService.Stub,
            _apis.SchemeService.ListDirectory,
            _wrap_list_directory_response,
            settings,
        )

    def async_describe_path(self, path, settings=None):
        return self._driver.future(
            _describe_path_request_factory(path),
            _apis.SchemeService.Stub,
            _apis.SchemeService.DescribePath,
            _wrap_describe_path_response,
            settings,
        )

    def async_modify_permissions(self, path, settings):
        """
        Modifies permissions for provided scheme entry

        :param path: A path of scheme entry
        :param settings: An instance of ModifyPermissionsSettings

        :return: An future of computation
        """
        return self._driver.future(
            _modify_permissions_request_factory(path, settings),
            _apis.SchemeService.Stub,
            _apis.SchemeService.ModifyPermissions,
            operation.Operation,
            settings,
        )
